#!/bin/perl
package cping;
use 5.6.0;        # should be tested with 5.0x as well

# author's constants
use constant CPING_APPNAME      => 'CPing';
use constant CPING_VERSION      => '0.9b';
use constant CPING_COPYRIGHT    => "\n@{[ CPING_APPNAME ]} @{[ CPING_VERSION ]}: " .
                                   "Cuneyt Yilmaz (progs-cping(\@)cuneytyilmaz.com), March 2003\n" .
                                   "http://www.cuneytyilmaz.com/prog/cping/\n" .
                                   "This program is distributed under GPL License\n" .
                                   "For details see: http://www.gnu.org/licenses/gpl.html\n";

use Net::Ping;
use Net::Domain qw(hostfqdn);
use Net::hostent qw(gethost);
use Socket qw(inet_ntoa);
#use File::Basename qw(fileparse);   #TODO, not needed
use Getopt::Long qw(GetOptions);
# the two lines below do not work all the times under Windows
#use sigtrap qw(handler break_handler INT
#               die untrapped old-interface-signals);
$SIG{INT} = \&cping::break_handler;     # off Perl Cookbook

use strict;


#`~..~`~..~`~..~`~..~`~..~


# get this script's name, base name
my  $basename = &cping::_get_my_name();

# get where this script runs
my	$fqdn_name = &Net::Domain::hostfqdn();


#`~..~`~..~`~..~`~..~`~..~

# define the output constants
use constant OUTPUT_HTML                => 'HTML';
use constant OUTPUT_CONSOLE             => 'CON';

# define output success constants
use constant SUCCESS_ALL                => 'ALL';
use constant SUCCESS_ANY                => 'ANY';
use constant SUCCESS_LAST               => 'LAST';

# define the config file name
use constant CONFIG_FILE_EXT            => '.ini';

# define the protocols array
# tried in this order, unless specified otherwise, only the 1st one is used
# see DEFAULT_PROTOCOL constant below
use constant ALL_PROTOCOLS              => qw(icmp udp tcp);

# define the default constants
use constant DEFAULT_HOST               => '';
use constant DEFAULT_LOOKUPS_ACTIVE     => 1;       # do reverse dns lookups?
use constant DEFAULT_WAIT               => 0.25;    # in seconds
use constant DEFAULT_MIN_WAIT           => 0.25;    # in seconds
use constant DEFAULT_PROTOCOL           => (ALL_PROTOCOLS)[0];
use constant DEFAULT_COUNTER            => 4;
use constant DEFAULT_BYTES              => 64;
use constant DEFAULT_TTL                => 128;     # in msec
use constant DEFAULT_TIMEOUT            => 2;       # in seconds
use constant DEFAULT_TO_ALL_PROTOCOLS   => 0;
use constant DEFAULT_TO_INFINITE        => 0;
use constant DEFAULT_TO_VERBOSE_MODE    => 0;
use constant DEFAULT_TO_DEBUG_MODE      => 0;
use constant DEFAULT_OUTPUT_MODE        => OUTPUT_CONSOLE;
use constant DEFAULT_SUCCESS_MODE       => SUCCESS_ANY;
use constant DEFAULT_AGENT_MODE         => 0;
use constant DEFAULT_READ_CONFIG        => 0;


# define the exit codes
use constant EXIT_HOST_IS_ALIVE         => 0;   # host is alive
use constant EXIT_DISP_USAGE            => 1;   # no parameters, usage is displayed
use constant EXIT_INVALID_PARAM         => 2;   # invalid parameter
use constant EXIT_HOST_NOT_FOUND        => 4;   # host not found
use constant EXIT_INVALID_IP            => 8;   # invalid ip address
use constant EXIT_HOST_NOT_REACHABLE    => 16;  # host not reachable or ping is disabled
use constant EXIT_INVALID_HOST_TYPE     => 32;  # not a valid ip address or host name
use constant EXIT_UNKNOWN_OUTPUT_MODE   => 64;  # unknown output mode :)
use constant EXIT_UNKNOWN_ERROR_CODE    => 128; # something bad happened!
use constant EXIT_ABORTED_BEFORE_FINISH => -1;  # Ctrl-C is pressed

my  @exit_codes = qw(
    EXIT_HOST_IS_ALIVE
    EXIT_DISP_USAGE
    EXIT_INVALID_PARAM
    EXIT_HOST_NOT_FOUND
    EXIT_INVALID_IP
    EXIT_HOST_NOT_REACHABLE
    EXIT_INVALID_HOST_TYPE
    EXIT_UNKNOWN_OUTPUT_MODE
    EXIT_UNKNOWN_ERROR_CODE
    EXIT_ABORTED_BEFORE_FINISH
);
my  @protocols = ();

#`~..~`~..~`~..~`~..~`~..~


# declare the parameter variables
my  $host           = DEFAULT_HOST;
my  $lookup_dns     = DEFAULT_LOOKUPS_ACTIVE;
my  $protocol       = DEFAULT_PROTOCOL;
my  @def_protocols  = (DEFAULT_TO_ALL_PROTOCOLS ? (ALL_PROTOCOLS) : (DEFAULT_PROTOCOL) );
my  $timeout        = DEFAULT_TIMEOUT;
my  $ttl            = DEFAULT_TTL;
my  $bytes          = DEFAULT_BYTES;
my  $counter        = DEFAULT_COUNTER;
my  $allprotocols   = DEFAULT_TO_ALL_PROTOCOLS;
my  $infinite       = DEFAULT_TO_INFINITE;
my  $wait           = DEFAULT_WAIT;
my  $verbose        = DEFAULT_TO_VERBOSE_MODE;
my  $debug          = DEFAULT_TO_DEBUG_MODE;
my  $exitcodes      = 0;
my  $out_mode       = DEFAULT_OUTPUT_MODE;
my  $success_mode   = DEFAULT_SUCCESS_MODE;
my  $agent_mode     = DEFAULT_AGENT_MODE;
my  $read_config    = DEFAULT_READ_CONFIG;
my  $quiet          = 0;
my  $version        = 0;
my  $help           = 0;



# get the parameters
&Getopt::Long::GetOptions(
    'host=s'                => \$host,
    'lookup_dns!'           => \$lookup_dns,
    'protocol=s'            => \@protocols,
    'timeout:i'             => \$timeout,
    'ttl:i'                 => \$ttl,
    'bytes:i'               => \$bytes,
    'counter:i'             => \$counter,
    'allprotocols|tryall!'  => \$allprotocols,
    'infinite'              => \$infinite,
    'wait:s'                => \$wait,
    'out:s'                 => \$out_mode,
    'success:s'             => \$success_mode,
    'agent!'                => \$agent_mode,
    'readconfig!'           => \$read_config,
    'quiet!'                => \$quiet,
    'verbose!'              => \$verbose,
    'debug!'                => \$debug,
    'exitcodes'             => \$exitcodes,
    'version'               => \$version,
    'help|usage|?'          => \$help
    );


# get the host to ping (-h is optional if used as the first parameter ;)
# however -h overrides the first given nonformal parameter
# e.g. <cping.pl> dummyhost -h somehost.domain.com -> dummyhost is ignored

# the method below is broken
#my  $temp   = shift || '';
#my  $host   = ($temp eq '-h' ? shift || DEFAULT_HOST : $temp);

#`~..~`~..~`~..~`~..~`~..~

# check if the required parameters (namely 'host') are given
unless($host)
{
    ($host = shift @ARGV || '') =~ s/^-+//; # remove empty options (e.g. 'cping -')
    $host = $host || DEFAULT_HOST;          # fallback host; should be normally not used
}

#`~..~`~..~`~..~`~..~`~..~

# if the user wants to see the exit codes or version, show'em and exit
if($exitcodes)
{
    &cping::disp_exit_codes();
    exit(EXIT_DISP_USAGE);
}
if($version)
{
    &cping::disp_version();
    exit(EXIT_DISP_USAGE);
}

if($help || !$host)
{
    &cping::disp_usage();
    exit(EXIT_DISP_USAGE);
}


#`~..~`~..~`~..~`~..~`~..~

# check conflicting options

# check output quietness, quiet overrides verbose & debug
($quiet, $debug, $verbose)  = ( $quiet ? (1, 0, 0) : (0, $debug, $verbose) );

# check output verbosity, debug overrides verbose
($debug, $verbose) = ( $debug ? (1, 1) : (0, $verbose) );


#`~..~`~..~`~..~`~..~`~..~

# TODO: a parser here?
# read the config file if exists, should be a Perl script!
my  $config_file_name;
if($read_config)
{
    $config_file_name = $basename . CONFIG_FILE_EXT;

    do "$ENV{HOME}/" . $config_file_name
        or
    do "$ENV{HOMEDRIVE}$ENV{HOMEPATH}/" . $config_file_name
        or
    do "$ENV{USERPROFILE}/" . $config_file_name
        or
    print "can't find any user config file: $config_file_name\n" if($debug);
    print "Searched\n" if($debug);
    print '%HOME: ('. ( $ENV{HOME} ? $ENV{HOME} : '') . ")\n" if($debug);
    print '%HOMEDRIVE%HOMEPATH: (' . ( $ENV{HOMEDRIVE}.$ENV{HOMEPATH} ? $ENV{HOMEDRIVE}.$ENV{HOMEPATH} : '') . ")\n" if($debug);
    print '%USERPROFILE: (' . ( $ENV{USERPROFILE} ? $ENV{USERPROFILE} : ''). ")\n" if($debug);
}

#`~..~`~..~`~..~`~..~`~..~

# agent mode automatically activates infinite mode, thus has the least priority
($agent_mode, $infinite) = ( $agent_mode ? (1, 1) : (0, $infinite) );

# TODO: is this restriction necessary?
# check allprotocols (try all protocols one by one) level, allprotocols overrides infinite and protocol
# if not all are tried, use the given protocols, else defaults
($infinite, @protocols) = (
    $allprotocols ?
    (0, ALL_PROTOCOLS) :
    ($infinite, (@protocols ? @protocols : @def_protocols) )
    );

# TODO: is this restriction necessary?
# in infinite mode, use only the first protocol
@protocols = ($infinite ? ($protocols[0]) : @protocols);


# 'correct' the options if necessary

# if no counter is given, set to default
$counter = ($counter ? $counter : DEFAULT_COUNTER);

# if in loop mode, set infinite to 1
$infinite = ($infinite ? 1 : 0);

# if wait not given, set to default
$wait   = ($wait ? $wait : DEFAULT_WAIT);

# if it's less than minimum allowed, set it to min
$wait   = ($wait < DEFAULT_MIN_WAIT ? DEFAULT_MIN_WAIT : $wait);

# convert to miliseconds
#$wait   *= 1000;

# check output mode, CONSOLE or HTML output, if not given, use default
$out_mode       = ($out_mode =~ /^(CON|HTML)$/i ? uc($out_mode) : DEFAULT_OUTPUT_MODE);

# check success mode, all should succeed?, any should succeed?, last result?
$success_mode   = ($success_mode =~ /^(ALL|ANY|LAST)$/i ? uc($success_mode) : DEFAULT_SUCCESS_MODE);


#`~..~`~..~`~..~`~..~`~..~


# check output type and start it accordingly
$out_mode = ( $out_mode eq OUTPUT_HTML ? OUTPUT_HTML : OUTPUT_CONSOLE );
print "Content-type: text/html\n\n" if ($out_mode eq OUTPUT_HTML);


#`~..~`~..~`~..~`~..~`~..~

# parameters debug information
print qq 

Using parameters
host         : $host
lookup dns   : $lookup_dns
protocol(s)  :  . (join (', ', @protocols)) . qq 
timeout      : $timeout
ttl          : $ttl
bytes        : $bytes
counter      : $counter
allprotocols : $allprotocols
infinite     : $infinite
wait         : $wait
out          : $out_mode
success      : $success_mode
agent mode   : $agent_mode
config       : $read_config (config file: $config_file_name)
quite mode   : $quiet
verbose mode : $verbose
debug mode   : $debug

 if ($debug);


#`~..~`~..~`~..~`~..~`~..~


# check if the given host is an FQDN or an IP Address
my  ($host_obj, $ip_addr, $host_name);
if( &cping::valid_ip_address($host) )
{
    if($lookup_dns)
    {
        $host_obj   = Net::hostent::gethost($host);
        unless($host_obj)
        {
            print uc($host) . ": IP address is well-formed but could not be resolved!\n" unless ($quiet);
            exit(EXIT_HOST_NOT_FOUND);
        }
        $host_name  = $host_obj->name;
        $ip_addr    = $host;
    }
}
elsif( &cping::valid_host_name($host) )
{
    if($lookup_dns)
    {
        $host_obj   = Net::hostent::gethost($host);
        unless($host_obj)
        {
            print uc($host) . ": Host name is well-formed but could not be resolved!\n" unless ($quiet);
            exit(EXIT_HOST_NOT_FOUND);
        }
        $host_name  = $host;
        $ip_addr    = inet_ntoa($host_obj->addr);
    }
}
else
{
    print uc($host) . ": Cannot determine type of given host! IP or host name is invalid!\n" unless ($quiet);
    exit(EXIT_INVALID_HOST_TYPE);
}

print "IP address: $ip_addr <=> FQDN: $host_name\n" if($verbose);
print "Success mode: $success_mode".($agent_mode ? ', running in Agent Mode' : '')."\n\n" if($verbose);


#`~..~`~..~`~..~`~..~`~..~

# initialize the stats variables
my  $cnt            = 0;
my  $cnt_fail       = 0;
my  $cnt_succ       = 0;
my  $cnt_displayed  = 0;
my  $last_exit_code = undef;


# print header
print "PING "
	. ($host eq $ip_addr ? $ip_addr : "$host ($ip_addr)" )
	. ($fqdn_name ? " from $fqdn_name" : '' )
	. " : $bytes bytes of data.\n"
unless($quiet);


# create the ping object(s) with the given parameters
# all three objects are created in advance, in order to avoid the overhead in the loop below
my  $p_tcp  = Net::Ping->new('tcp',  $timeout, $bytes)
    or die "Can't create new tcp-ping object:\n$!\n" unless($quiet);
my  $p_udp  = Net::Ping->new('udp',  $timeout, $bytes)
    or die "Can't create new udp-ping object:\n$!\n" unless($quiet);
my  $p_icmp = Net::Ping->new('icmp', $timeout, $bytes)
    or die "Can't create new icmp-ping object:\n$!\n" unless($quiet);

while($cnt < $counter)
{
    # in loop mode do not increment the counter, so that it runs forever
    unless($infinite) { $cnt++; }

    foreach my $prot (@protocols)
    {
        #print "$prot\n";
        #$p_obj = (eval "p_$prot");
        #print "obj_name: $p_obj\n";

        # ping the given host
        my  $ping_res = eval "\$p_$prot->ping(\"$ip_addr\", $timeout);";

        # return the results
        if( $ping_res )
        {
            ++$cnt_succ;
            print "$bytes bytes from $host_name ($ip_addr): " . $protocol . "_seq=$cnt_displayed timeout=" . ($timeout*1000) . " msec\n"  unless($quiet);
            $last_exit_code = EXIT_HOST_IS_ALIVE;

            if( $agent_mode and
                    ($success_mode eq SUCCESS_LAST or $success_mode eq SUCCESS_ANY)
              )
            {
                # i.e. ping until the host is up
                # increase counter to the limit
                print "Given host is up! Agent mode succeeded, now exiting!\n";
                $cnt = $counter;
            }
        }
        else
        {
            ++$cnt_fail;
            print uc($host) . ": Host not reachable via ". uc($prot) ." or PING is disabled on the host!\n" unless($quiet);
            $last_exit_code = EXIT_HOST_NOT_REACHABLE;
            if( $agent_mode and ($success_mode eq SUCCESS_ALL ))
            {
                # i.e. ping until the host is down
                # do the cleanup and exit
                print "Given host is down! Agent mode succeeded, now exiting!\n";
                $cnt = $counter;
            }
        }
    }

    # wait as specified
    if($counter > 1) { select(undef, undef, undef, $wait); }
    ++$cnt_displayed;
}



# destroy the ping objects
$p_tcp->close;
$p_udp->close;
$p_icmp->close;



# compute the statistics and display them
my  $cnt_total = $cnt_fail + $cnt_succ;
my  $loss = int (100 * $cnt_fail / $cnt_total);
print qq 
--- $host ping statistics ---
$cnt_total packets transmitted, $cnt_succ packets received, $loss\% packet loss
 unless($quiet);


# compute the exit code
my  $exit_code  = EXIT_UNKNOWN_ERROR_CODE;

if($success_mode eq SUCCESS_LAST)       # exit with the last exit code returned
{
    $exit_code  = $last_exit_code;
}
elsif($success_mode eq SUCCESS_ANY)     # succesful if any were succesful
{
    $exit_code = ($cnt_succ ? EXIT_HOST_IS_ALIVE : EXIT_HOST_NOT_REACHABLE);
}
elsif($success_mode eq SUCCESS_ALL)     # failed if any failed (= succesful only if all were succesful)
{
    $exit_code = ($cnt_fail ? EXIT_HOST_NOT_REACHABLE : EXIT_HOST_IS_ALIVE);
}



print "Exit code: $exit_code\n" if($verbose);
exit($exit_code);



#`~..~`~..~`~..~`~..~`~..~








#`~..~`~..~`~..~`~..~`~..~
#
#       F U N C T I O N    D E C L A R A T I O N S
#
#`~..~`~..~`~..~`~..~`~..~
sub break_handler
{
    $cnt = 0;
    print "Aborting\n";
    #print "exit_code: ". $testvar ."\n";
    exit (EXIT_ABORTED_BEFORE_FINISH);
}
#`~..~`~..~`~..~`~..~`~..~
sub valid_ip_address
{
    my  $ip = shift;

    # return if first or last number is 0
    return 0 if( $ip =~ /^0\./ || $ip =~ /\.0$/ );

    # from perl cookbook, but corrected :)
    return ( $ip =~ /^ ( [01]?\d\d? | 2[0-4]\d | 25[0-5] )
                        \.
                       ( [01]?\d\d? | 2[0-4]\d | 25[0-5] )
                       \.
                       ( [01]?\d\d? | 2[0-4]\d | 25[0-5] )
                       \.
                       ( [01]?\d\d? | 2[0-4]\d | 25[0-5] )
                     $/x
            ? 1 : 0 );
}
#`~..~`~..~`~..~`~..~`~..~
sub valid_host_name
{
    my  $hn = shift;

    # if the address is beginning with a number, or any word, it should be(?) invalid
    return ( $hn =~ /(^\d|\.\d)/ ? 0 : 1 );
}
#`~..~`~..~`~..~`~..~`~..~
sub is_in_array
{
    my  $needle     = shift;
    my  $no_case    = shift || 0;
    return unless($needle);

    foreach my $hay(@_)
    {
        if($no_case)
        {
            return 1 if( uc($needle) eq uc($hay) );
        }
        else
        {
            return 1 if( $needle eq $hay );
        }
    }
    return;
}
#`~..~`~..~`~..~`~..~`~..~
sub disp_usage
{

    print CPING_COPYRIGHT;
    print qq 
Usage : @{[ CPING_APPNAME ]}
        [[--host] hostname]
        [--protocol[=]icmp|tcp|udp] [--allprotocols|tryall]
        [--timeout[=]seconds] [--ttl[=]seconds] [--bytes[=]num_of_bytes]
        [--counter[=]times_to_ping] [--infinite]
        [--wait[=]seconds_to_wait] [--out[=]html|con]
        [--success[=]any|all|last]
        [--config|--noconfig]
        [--quiet|--noquiet] [--verbose|--noverbose] [--debug|--nodebug]
        [--exitcodes] [--version] [--help|usage]
--host                : hostname or IP address to ping
                        optional, first non-param will be used
--lookup_dns          : activate reverse dns lookups
--protocol            : can be one of (ICMP, TCP, UDP)
--timeout             : timeout in seconds
--ttl                 : time-to-live in seconds
--bytes               : number of bytes to be sent in each ping packet
--counter             : number of times to ping (infinite overrides this)
--allprotocols|tryall : try all protocols one after another (overrides infinite)
--infinite            : ping until user interrupts (Ctrl-C is catched)
--wait                : seconds to wait between each request
--out                 : output mode; can be one of (HTML, CON)
--success             : determine exit code according to
                        (ANY | LAST | ALL) successful ping result(s)
--agent               : activate agent mode, only with success
                        with ALL     : ping until host is down
                        with LAST|ANY: ping until host is up
--readconfig          : do load config file, should be a Perl script
--noconfig            : do not load config file
--quiet               : do not output anything (overrides verbose & debug)
--noquiet             : do output default messages
--verbose             : show more information
--noverbose           : show default messages
--debug               : show every debug message produced
--nodebug             : show no debug messages
--exitcodes           : list the exit codes produced by the program and exit
--version             : show version information and exit
--help|usage|?        : show this screen and exit

All parameters can be shortened, as long as no ambiguity occurs
e.g. --allprotocols, -allprotocols, --allprot, -all are all the same
but  --h causes ambiguity since there is --help & --host

;
    exit(EXIT_DISP_USAGE);
}
#`~..~`~..~`~..~`~..~`~..~
sub disp_exit_codes
{
    print CPING_COPYRIGHT;
    print "\nExit codes for shell scripts:\n\n";
    foreach my $cur_exit_code (@exit_codes)
    {
        print sprintf("%3s", (eval "$cur_exit_code")) . " ($cur_exit_code)\n";
    }

    exit(EXIT_DISP_USAGE);
}
sub disp_version
{
    print CPING_APPNAME . ' ' . CPING_VERSION;
}
#`~..~`~..~`~..~`~..~`~..~


# returns the filename and extension parts off this file's name
sub _get_my_name
{
    # get the whole path
    my  $orig_name    = $0;
    #get after last / or \
    $orig_name =~ m!^(?:.*)[\\/]([^\\/]*)\.([^\.\\/]*)$!i;
    return wantarray ? ($1, $2) : $1;

    # alternative solution
    #$my_name = &File::Basename::basename($0);
}

__END__

#`~..~`~..~`~..~`~..~`~..~
#
# S A M P L E     O U T P U T S     F R O M     U N I X     &     W I N D O W S
#
#`~..~`~..~`~..~`~..~`~..~

#  UNIX OUTPUT
#
#  PING www.yahoo.akadns.net (66.218.71.81) from 12.37.166.67 : 56(84) bytes of data.
#  64 bytes from w2.scd.yahoo.com (66.218.71.81): icmp_seq=0 ttl=51 time=68.300 msec
#  64 bytes from w2.scd.yahoo.com (66.218.71.81): icmp_seq=1 ttl=51 time=69.924 msec
#  64 bytes from w2.scd.yahoo.com (66.218.71.81): icmp_seq=2 ttl=51 time=105.567 msec
#  64 bytes from w2.scd.yahoo.com (66.218.71.81): icmp_seq=3 ttl=51 time=83.190 msec
#
#  --- www.yahoo.akadns.net ping statistics ---
#  4 packets transmitted, 4 packets received, 0% packet loss
#  round-trip min/avg/max/mdev = 68.300/81.745/105.567/14.918 ms



#  WINDOWS OUTPUT
#
#  Pinging localhost [127.0.0.1] with 32 bytes of data:
#
#  Reply from 127.0.0.1: bytes=32 time<10ms TTL=128
#  Reply from 127.0.0.1: bytes=32 time<10ms TTL=128
#  Reply from 127.0.0.1: bytes=32 time<10ms TTL=128
#  Reply from 127.0.0.1: bytes=32 time<10ms TTL=128
#
#  Ping statistics for 127.0.0.1:
#      Packets: Sent = 4, Received = 4, Lost = 0 (0% loss),
#  Approximate round trip times in milli-seconds:
#      Minimum = 0ms, Maximum =  0ms, Average =  0ms
#




=pod

=head1 NAME

CPing - ping a remote host

=head1 SYNOPSIS

cping localhost --allprotocols --verbose --protocol tcp
        --success ANY --out CON

=head1 DESCRIPTION

This is a simple perl wrapper script for the Net::Ping module
adding some functionality, in order to be use it from another
script. The motivation behind it was to be able to ping remote
hosts and to take some actions according to the exit codes.

For simple server monitoring, see success and agent parameters.

Most of the parameters are self-explanatory. For reference
here is a quick summary of command line parameters:

=over 4

=item
--host                : IP name or hostname to ping

=item
--lookup_dns          : activate reverse DNS lookups

=item
--protocol            : can be one of (ICMP, TCP, UDP)

=item
--timeout             : timeout in seconds

=item
--ttl                 : time-to-live in seconds

=item
--bytes               : number of bytes to be sent in each ping packet

=item
--counter             : number of times to ping (infinite overrides this)

=item
--allprotocols|tryall : try all protocols one after another (overrides infinite)

=item
--infinite            : ping until user interrupts (Ctrl-C is catched)

=item
--wait                : seconds to wait between each request

=item
--out                 : output mode; can be one of (HTML, CON)

=item
--success             : determine exit code according to
                        (ANY | LAST | ALL) successful ping result

=item
--agent_mode          : activate agent mode
                        with ALL     : ping until host is down
                        with LAST|ANY: ping until host is up

=item
--readconfig          : do load config file, should be a perl script

=item
--noconfig            : do not load config file

=item
--quiet               : do not output anything (overrides verbose & debug)

=item
--noquiet             : do output default messages

=item
--verbose             : show more information

=item
--noverbose           : show default messages

=item
--debug               : show every debug message produced

=item
--nodebug             : show no debug messages

=item
--exitcodes           : list the exit codes produced by the program and exit

=item
--version             : show version information and exit

=item
--help|usage|?        : show help screen and exit

=item

=back

=head1 EXIT CODES

This program returns some exit codes upon finishing. These are
called in Unix world exit codes and in DOS world errorlevels.
Please refer to your OS' manual for the usage details.

=over 4

=item
EXIT_HOST_IS_ALIVE         => 0

=item
EXIT_DISP_USAGE            => 1;

=item
EXIT_INVALID_PARAM         => 2;

=item
EXIT_HOST_NOT_FOUND        => 4;

=item
EXIT_INVALID_IP            => 8;

=item
EXIT_HOST_NOT_REACHABLE    => 16;

=item
EXIT_INVALID_HOST_TYPE     => 32;

=item
EXIT_UNKNOWN_OUTPUT_MODE   => 64;

=item
EXIT_UNKNOWN_ERROR_CODE    => 128;

=item
EXIT_ABORTED_BEFORE_FINISH => -1;

=item

=back

=head1 INSTALLATION

Just copy this script somewhere in your PATH variable.
Also make sure that:
- either .pl files can be executed in your DOS box,
- or the path for perl in the first line of this script
  points to the correct perl binary.

=head1 KNOWN BUGS

If the reverse lookups fail, this programs can produce
false results.

And the error handling mechanism does not work as intended.

=head1 HAILS

All hails to developers of Mozilla, the one and only real
browser at the moment.

Without my Rush & Coroner CD's I would be bored to death
writing this script.


=head1 COPYRIGHT

Copyright 2003, Cuneyt Yilmaz (C&uuml;neyt Y&yacute;lmaz).
All rights reserved.

This program is distributed under the same conditions
and terms as GNU Public License, GPL. For details,
see: http://www.gnu.org/licenses/gpl.html

=cut
